#include "mpcgen.h"
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>


void summarize( struct client_data *clients, int max, time_t curr_time );

int usage( char *prog )
{
	printf( "usage: %s\n", prog );
	printf( "listens for traffic to mcast_addr on port\n" );
        printf( "options:\n" );
        printf( "-m mcast_addr\n" );
        printf( "-p port\n" );
        printf( "-s pkt_size\n" );
        printf( "-c packets_per_second\n" );
        printf( "-n max_clients\n" );
	exit(2);
}

int main( int argc, char *argv[] )
{
	struct client_data *clients;
	struct client_data *curr_client;

	struct message *msg;
	int sd;
	time_t curr_time;


	int num;
	int len;
	struct sockaddr_in csockaddr;
	int client_naddr;
	int client_port;

	fd_set read_set, all_set;
	struct timeval tv;

	struct options opts;

        opts.ttl = 16;
        opts.port = 60000;
        opts.pkts_per_sec = 10;
        opts.pkt_size = 1500;
        opts.burst = 0;
        opts.window_size = 1000;
        opts.max_clients = 16;
	strcpy( opts.mcast_addr, "239.70.70.1" );


	tv.tv_sec = 1;
	tv.tv_usec = 0;

	len = sizeof(csockaddr);

	clients = (struct client_data *) malloc ( clients, sizeof(struct client_data) * opts.max_clients );
	bzero( clients, sizeof(struct client_data) * opts.max_clients );

	if ( opts.pkt_size < sizeof( struct message ) + IP_HEADER_SIZE )
	{
		opts.pkt_size = sizeof( struct message ) + IP_HEADER_SIZE;
	}

	msg = (struct message *) malloc( opts.pkt_size - IP_HEADER_SIZE );


	sd = open_listener_socket( opts.mcast_addr, opts.port );

	
	curr_time = time( NULL );

	printf ("listening for traffic to %s port %d pkt_size %d\n", opts.mcast_addr, opts.port,  opts.pkt_size );
	while(1)
	{
		if( curr_time < time(NULL) )
		{
			summarize( clients, opts.max_clients, curr_time );
			curr_time = time( NULL );
		}

		FD_ZERO( &read_set );
		FD_SET( sd, &read_set );

		num = select( sd + 1, &read_set, NULL, NULL, &tv );
		if ( num < 0 )
		{
			printf ( "error with select call\n" );
			continue;
		}
		if ( num == 0 )
		{
			continue;
		}
		
		num = recvfrom(sd, msg, opts.pkt_size - IP_HEADER_SIZE, 0, (struct sockaddr *) &csockaddr,&len);
		if( num<0 )
		{
			printf ( "failed to recvfrom socket\n" );
			continue;
		}

		client_naddr = csockaddr.sin_addr.s_addr;
		client_port = ntohs(csockaddr.sin_port);



		curr_client = find_client( client_naddr, clients, opts.max_clients );
		if ( curr_client == NULL )
		{
			/* find a free one */
			curr_client = find_client( 0, clients, opts.max_clients );
			if ( curr_client == NULL )
			{
				printf ( "failed to find free client data struct\n" );
				continue;
			}
			curr_client->naddr = client_naddr;
			curr_client->port = client_port;
			strcpy( curr_client->addr, inet_ntoa( csockaddr.sin_addr ) );
			curr_client->addr[16]='\0';
			curr_client->first_seq = msg->seq_number;
		}

		curr_client->recieved_pkts++;
		curr_client->pkts_per_sec =  msg->pkts_per_sec;

		if ( curr_client->last_seq + 1 < msg->seq_number  )
		{
			curr_client->early_pkts++;
			curr_client->last_seq = msg->seq_number;
		}
		if ( curr_client->last_seq >  msg->seq_number )
		{
			curr_client->late_pkts++;
		}
		if ( curr_client->last_seq + 1 == msg->seq_number )
		{
			curr_client->last_seq = msg->seq_number;
		}
	}
}

void summarize( struct client_data *clients, int max, time_t curr_time )
{
	int i;
	struct tm *ltime;
	struct client_data *curr_client;

	ltime = localtime(&curr_time);
	for ( i = 0; i < max; i++ )
	{
		curr_client = &clients[i];

		if ( curr_client->naddr == 0 ) { continue; }

		printf ("%02d:%02d:%02d ", ltime->tm_hour, ltime->tm_min, ltime->tm_sec );
		printf ( "%llu pkts of %llu from %s port %llu : %llu early %llu late (seq %llu thru %llu)\n", (unsigned long long) curr_client->recieved_pkts, 
										(unsigned long long) curr_client->pkts_per_sec,
										curr_client->addr,
										(unsigned long long) curr_client->port,
										(unsigned long long) curr_client->early_pkts,
										(unsigned long long) curr_client->late_pkts,
										(unsigned long long) curr_client->first_seq,
										(unsigned long long) curr_client->last_seq );

		curr_client->recieved_pkts = 0;
		curr_client->early_pkts = 0;
		curr_client->late_pkts = 0;
		curr_client->first_seq = curr_client->last_seq + 1;
	}

}
