#include "mpcgen.h"
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>



int usage( char *prog )
{
	printf( "usage: %s [options]\n", prog );
	printf( "sends sequenced datagrams to mcast_addr on port\n" );
	printf( "options:\n" );
	printf( "-m mcast_addr\n" );
	printf( "-p port\n" );
	printf( "-s pkt_size\n" );
	printf( "-c packets_per_second\n" );
	printf( "-b burst_count (sends packets in timed bursts)\n" );
	printf( "-t ttl\n" );
	exit(2);
}

int main( int argc, char *argv[] )
{
	int sd;
	struct options opts;
	struct message *msg;

	int num;
	int i;
	int seq;
	int len;
	time_t curr_time;
	uint64_t count;
	uint64_t first_seq;

	struct timespec delay;

	struct tm *ltime;

	struct sockaddr_in dest;

	opts.ttl = 16;
	opts.port = 60000;
	opts.pkts_per_sec = 10;
	opts.pkt_size = 1500;
	opts.burst = 0;
	opts.window_size = 1000;
	opts.max_clients = 16;
	strcpy( opts.mcast_addr, "239.70.70.1" );


	get_options( argc, argv, &opts );

	sd = open_sender_socket( 0, opts.ttl );

	setup_dest_addr( &dest, opts.mcast_addr, opts.port );

	delay.tv_sec = 0;

	if ( opts.pkts_per_sec < opts.burst )
	{
		opts.burst = 0;
	}
	if ( opts.burst > 0 )
	{
		delay.tv_nsec = (long) 1000000000 / (long) (  opts.pkts_per_sec / opts.burst  ) ;
	}

	if ( opts.pkt_size < sizeof( struct message ) + IP_HEADER_SIZE )
	{
		opts.pkt_size = sizeof( struct message ) + IP_HEADER_SIZE;
	}

	msg = (struct message *) malloc( opts.pkt_size - IP_HEADER_SIZE );

	msg->seq_number = 1;
	msg->pkts_per_sec = opts.pkts_per_sec;

	printf( "msg size is %d\n", opts.pkt_size );
	if ( opts.burst != 0 )
	{
		printf( "sending pkts in bursts of %d\n", opts.burst );
		printf( "inter-burst delay is %d nsecs\n", delay.tv_nsec );
	}

	curr_time = time(NULL);
	count = 0;
	first_seq = 1;

	while(1)
	{
		if (  count < opts.pkts_per_sec )
		{
			num = sendto( sd, msg, opts.pkt_size - IP_HEADER_SIZE, 0 , (struct sockaddr *) &dest, sizeof(dest) );
			msg->seq_number++;
			count++;
		}
		if( num<0 )
		{
			continue;
		}
		if ( curr_time < time(NULL) )
		{
			ltime = localtime(&curr_time);
			printf ("%02d:%02d:%02d ", ltime->tm_hour, ltime->tm_min, ltime->tm_sec );

			printf ( "sent %lld pkts to %s port %d (seq %lld thru %lld)\n", count, opts.mcast_addr, opts.port, first_seq, msg->seq_number - 1 );
			count = 0;
			first_seq = msg->seq_number;
			curr_time = time(NULL);
		}
		if ( opts.burst != 0 && count % opts.burst  == 0 )
		{
			nanosleep( &delay, NULL );
		}
	}


	close(sd);


}
